# autodownload script
import urllib2
import urllib
import zipfile
import time
import os
from cStringIO import StringIO
import es

#plugin information
info = es.AddonInfo() 
info.name     = 'es_install: Automatic installer for ES' 
info.version  = '1.0' 
info.url      = 'http://mattie.info/cs' 
info.basename = 'es_install' 
info.author   = 'Errant/Mattie'

# ========================
# Initial constants
# ========================            
BASE_URL =  'http://addons.eventscripts.com'
INSTALL_PATH = str(es.ServerVar('eventscripts_gamedir'))
DOWNLOAD_PATH = str(es.ServerVar('eventscripts_addondir')) + "/corelib/es_install/temp/"

'''
== 1.0 ==
Released to SVN
== 1.0RC2 ==
Slight updates and tweaks, planned final release candidate
- [+] Added es_xinstall and es_xuninstall in line with it being a corelib addition
- [+] Added autoload addons code (to exec es_install created cfg files in the autoload directory)
- [+] Added some logging to the install code
- [CHNG] Minor cosmetic tweaks to code
- [CHNG] Updated paths to new corelib locations
== 1.0RC ==
First release candidate
- [+] Added uninstall option alongide install (es_uninstall)
- [+] Pushed all the relevant methods into a class to make installation handling easier (esInstall())
- [+] Added cleanTempDir method to clean out .zips from the temporary download directory
- [+] Added error output to installTests class for better bebugging
- [+] Added a public method to return an instance of esInstall for an addon
- [CHNG] esInstall->checkInstalled() now returns TRUE if an addon is installed (previosuly it returned false)
- [CHNG] Converted install tests to use the new class method
- [CHNG] Reformated code and cleaned it up somewhat
== 0.5 ==
- [+] Added try / except to isScriptApproved() method to catch url's that cannot be opened (ie invalid ones)
- [FIX] Fixed InstallTests->clean() failing to remove addon file
- [FIX] uninstallZip->uninstallFromZip() now closes the zip archive pointer after use (to allow deletion)
- [FIX] Fixed InstallTests->testSiteUrl() failing due to a non existant variable
- [FIX] Fixed InstallTests->testWrongSiteApproved() not resetting the url correctly
- [FIX] Fixed InstallTests->testDownloadingFile() trying to grab a path from an unset variable in several places
- [FIX] Fixed InstallTests->testFullInstall() trying to use an unset variable
- [CHNG] Changed caps'd ES_install to es_install
- [Incremented V#] First full working version - nearly ready for release 
== 0.4.1 ==
- [FIX] Minor fixes in the test class to make it work
== 0.4 ==
- [+] Added file replace warning
- [+] Unit testing & cleanup code added (es_install_test console command)
== 0.3.1 ==
- [+] es_install <x> autoload now creates an autoload file in eventscripts/_autoload
== 0.3 ==
- [+] Working as console command
- [FIX} Fixed errors trying to remove the temporary zip once installed (due to not closign it in addonZip)
== 0.2.1 ==
- Tweaked so as not to subclass zipfile.ZipFile (at matties request)
== 0.2 ==
- Reworked as an es addon
- [+] Unzipping / extraction code
- [+] checkInstalled to check for addons existence
- [FIX] isScriptApproved() no longer errors if the url returns somethign unexpected (returns False instead)
== 0.1 ==
- Matties original code
'''

# ========================
# Private classes
# ========================            
class addonZip(object):
    '''
    An extension of ZipFile to extract files to the right directories
    '''
    def __init__(self, filename, dir):
        '''
        Inits a zipfuile object for us to play with
        '''
        self.zip = zipfile.ZipFile(filename)
        self.dir = dir
    def extract(self):
        '''
        Grabs the list of files / dirs
        Creates a directory structure and then reads the files into it.
        '''
        overwrite_warning = 0
        dir = self.dir
        nameslist = self.zip.namelist()
        # filter the names from our list of dirs / files
        dirlist = filter( lambda x: x.endswith( '/' ), nameslist )
        filelist = filter( lambda x: not x.endswith( '/' ), nameslist )
        # check the base dir exists
        cwd = os.getcwd()
        if not os.path.isdir(dir):
            os.mkdir(dir)
        os.chdir(dir)
        # create directory structure
        dirlist.sort()
        es.dbgmsg(1, "== Creating directory structure ==")
        for i, dirs in enumerate(dirlist):
            dirs = dirs.split('/')
            prefix = ''
            for dir in dirs:
                dirname = os.path.join(prefix, dir)
                if dir and not os.path.isdir( dirname ):
                    os.mkdir( dirname )
                prefix = dirname
        es.dbgmsg(1, "Created",i,"directories")
        # extract files
        file_count = len(filelist)
        es.dbgmsg(1, "== Extracting files ==")
        for i, file in enumerate(filelist):
            try:
                if os.path.exists(file):
                    es.dbgmsg(0, "Warning: overwriting " + file)
                    overwrite_warning = 1
                target = open(file, 'wb')
                # read the zipped file into a buffer (useful as it drags it totally away from the zip)
                buffer = StringIO(self.zip.read(file))
                # read the first line.....
                line = buffer.read((2 ** 20))
                while line:
                    target.write(line)
                    line = buffer.read((2 ** 20))
                target.close()
            finally:
                info = "Extracted: (" + str(((i + 1) * 100) / file_count) + "%) " + file
                es.dbgmsg(1, info)
                
        if overwrite_warning:
            es.dbgmsg(0, "[es_install] ##################################################")
            es.dbgmsg(0, "[es_install] ### Note, some of the files on your server were ")
            es.dbgmsg(0, "[es_install] ### overwritten by this installation. This is")
            es.dbgmsg(0, "[es_install] ### probably because they are newer versions.")
            es.dbgmsg(0, "[es_install] ### ")
            es.dbgmsg(0, "[es_install] ### HOWEVER, it is worth checking this is the case")
            es.dbgmsg(0, "[es_install] ###################################################")
        self.zip.close()
        os.chdir(cwd)

class uninstallZip(addonZip):
    '''
    Extends addonzip to uninstall an addon
    '''
    def uninstallFromZip(self):
        '''
        Uses the zip file to remove all the files it installed...
        ~ however it will only install those in addons/eventscripts/<bbasename>/, sound/ and /cfg/
        '''
        dir = self.dir
        nameslist = filter( lambda x: x.startswith('addons/eventscripts/' + self.basename) or x.startswith('cfg/') or x.startswith('sound/'), self.zip.namelist())
        # filter the names from our list of dirs / files
        dirlist = filter( lambda x: x.endswith( '/' ), nameslist )
        filelist = filter( lambda x: not x.endswith( '/' ), nameslist )
        cwd = os.getcwd()
        if not os.path.isdir(dir):
            os.mkdir(dir)
        os.chdir(dir)
        # create directory structure
        dirlist.sort()
        dirlist.reverse()
        for file in filelist:
            if os.path.exists(file):
                os.remove(file)
        for dir in dirlist:
            if os.path.exists(dir):
                if not os.listdir(dir):
                    os.rmdir(dir)
        self.zip.close()
        os.chdir(cwd)
        
class esInstall(object):
    '''
    Addon Installation class
    '''
    # init some variables
    # init some variables
    basename        = None
    baseurl         = BASE_URL
    autoload        = None
    installpath     = INSTALL_PATH
    temppath        = DOWNLOAD_PATH
    dl_location     = None
    def __init__(self, basename, autoload=None,baseurl=None):
        self.basename=basename
        if autoload:
            self.autoload=autoload
        if baseurl:
            self.baseurl=baseurl
    def checkInstalled(self):
        '''
        Check if the addon is already installed (simply by checking the existence of the main file)
         - Returns true if it exists
        '''
        checkpathpy = os.path.join(es.getAddonPath(self.basename), self.basename + '.py')
        checkpathes = os.path.join(es.getAddonPath(self.basename), 'es_' + self.basename + '.txt')
        if os.path.exists(checkpathpy) or os.path.exists(checkpathes):
            return True
        return False
    def getBaseUrl(self):
        '''
        Gets hold of the baseurl
        '''
        return self.baseurl
    def setBaseUrl(self,newsite):
        '''
        Sets the baseurl
        '''
        self.baseurl = newsite
    def isScriptApproved(self):
        '''
        Checks with ESAM that it is an approved addon, returns true if it is.
        '''
        response = None
        try:
            response = urllib2.urlopen(self.baseurl + "/addons/chkaprv/" + self.basename)
        except URLError:
            es.dbgmsg(0, "Error: " + self.baseurl + " is an invalid URL.. unable to continue.")
        finally:
            if response:
                data = response.read()
                # we SHOULD have only one character (1 or 0) on the page
                if len(data) == 1:
                    return bool(int(data))
                else:
                    # error with the page.... 
                    return False
            else:
                return False
    def downloadAddon(self):
        '''
        Downloads the addon and returns true if it is a zip file downloaded
        '''
        self.dl_location = self.temppath + self.basename + ".zip"
        addondl = urllib.urlretrieve(self.baseurl + "/addons/download/" + self.basename,self.dl_location)
        return "zip" in addondl[1]['content-type']
    def installFile(self):
        '''
        Extracts the file, cleans up the temporary zip stored in es_install/temp and checks the installation went ok
        '''
        if self.dl_location:
            if os.path.exists(self.dl_location):
                tzip = addonZip(self.dl_location, self.installpath)
                tzip.extract()
                os.remove(self.dl_location)
                # now quickly check it is installed ok
                if self.checkInstalled():
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False
    def uninstallFile(self):
        '''
        Extracts the file and deletes the addon based on that
        '''
        if self.dl_location:
            if os.path.exists(self.dl_location):
                tzip = uninstallZip(self.dl_location, self.installpath)
                tzip.basename=self.basename
                tzip.uninstallFromZip()
                os.remove(self.dl_location)
                # removethe autoload file if it iexists
                self.cleanAutoload()
                # now quickly check it is uninstalled
                if not self.checkInstalled():
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False
    def setAutoLoad(self):
        '''
        Sets the addon to be autoloaded
        '''
        fp = open(self.installpath + "/addons/eventscripts/_autoload/autoload_%s.cfg" % self.basename, "wb")
        fp.write("es_load " + self.basename)
        fp.close()
        return True
    def cleanAutoload(self):
        '''
        Cleans all the unit test files
        '''
        if os.path.exists(self.installpath + "/addons/eventscripts/_autoload/autoload_%s.cfg" % self.basename):
            os.remove(self.installpath + "/addons/eventscripts/_autoload/autoload_%s.cfg" % self.basename)
    def cleanTempDir(self):
        '''
        Cleans the temporary directory of files
        '''
        for files in os.listdir(self.temppath):
            #clean up our downloaded temp files
            os.remove(os.path.join(self.temppath, files))
    def install(self):
        '''
        Install the addon
        '''
        changelog("[es_install] Installing %s" % self.basename)
        es.dbgmsg(0, "[es_install] Checking for prior installation")
        if not self.checkInstalled():
            # is isnt installed so carry on
            es.dbgmsg(0, "[es_install] Checking %s is an approved addon" % self.basename)
            if self.isScriptApproved():
                # it's approved! yay
                if self.downloadAddon():
                    es.dbgmsg(0, "[es_install] Extracting & installing %s" % self.basename)
                    if self.installFile():
                        if self.autoload:
                            # place holder for autoload option
                            self.setAutoLoad()
                            es.dbgmsg(0, "[es_install] %s will now be automatically loaded every time your server starts" % self.basename)
                            es.dbgmsg(0, "[es_install] Restart your server to load the addon")
                            changelog("[es_install] %s will autoload when your server starts" % self.basename)
                        es.dbgmsg(0, "[es_install] %s installed successfully" % self.basename)
                        es.dbgmsg(0, "[es_install] You can now use es_load %s to load the script" % self.basename)
                        changelog("[es_install] %s was successfully installed" % self.basename)
                    else:
                        es.dbgmsg(0, "Error: installation failed")
                        changelog("[es_install] %s was not installed successfully" % self.basename)
                        # clean up the temp file
                        if os.path.exists(self.dl_location):
                            self.cleanTempDir()
                            es.dbgmsg(0, "[es_install] Cleaned up temporary files & exiting")
                else:
                    es.dbgmsg(0, "Error: %s could not be downloaded from ESAM" % self.basename)
                    changelog("[es_install] %s was not installed successfully" % self.basename)
            else:
                es.dbgmsg(0, "Error: %s has not been approved by an ESAM moderator. Installation cannot continue" % self.basename)
                changelog("[es_install] %s was not installed successfully" % self.basename)
        else:
            es.dbgmsg(0, "Error: %s is already installed, please use es_update %s instead." % (self.basename, self.basename))
            changelog("[es_install] %s was not installed successfully" % self.basename)
    def uninstall(self):
        '''
        Uninstall the addon
        '''
        es.dbgmsg(0, "[es_install] Checking %s is installed" % self.basename)
        if self.checkInstalled():
            # is isnt installed so carry on
            es.dbgmsg(0, "[es_install] Checking %s is an approved addon" % self.basename)
            if self.isScriptApproved():
                # it's approved! yay
                if self.downloadAddon():
                    es.dbgmsg(0, "[es_install] Extracting %s" % self.basename)
                    if self.uninstallFile():
                        es.dbgmsg(0, "[es_install] ##################################################")
                        es.dbgmsg(0, "[es_install] ### Note: some of the files on your server may not")
                        es.dbgmsg(0, "[es_install] ### have been removed because they may be used by ")
                        es.dbgmsg(0, "[es_install] ### other addons.")
                        es.dbgmsg(0, "[es_install] ### ")
                        es.dbgmsg(0, "[es_install] ### It is worth manually checking all relevant")
                        es.dbgmsg(0, "[es_install] ### files have been deleted")
                        es.dbgmsg(0, "[es_install] ###################################################")
                        es.dbgmsg(0, "[es_install] %s uninstalled successfully" % self.basename)
                    else:
                        es.dbgmsg(0, "Error: uninstallation failed")
                        # clean up the temp file
                        if os.path.exists(self.dl_location):
                            os.remove(path)
                            es.dbgmsg(0, "[es_install] Cleaned up temporary files & exiting")
                else:
                    es.dbgmsg(0, "Error: %s could not be downloaded from ESAM" % self.basename)
            else:
                es.dbgmsg(0, "Error: %s has not been approved by an ESAM moderator. Uninstallation cannot continue" % self.basename)
        else:
            es.dbgmsg(0, "Error: %s is not installed" % self.basename)

class InstallTests(object):
    '''
    A set of unit tests
    '''
    def setUp(self):
        self.installer = esInstall("gungame4",1)
        self.mminstaller = esInstall("mugmod")
        self.gg99installer = esInstall("gungame99")
        self.temppath = str(es.ServerVar('eventscripts_addondir')) + "/es_install/temp/"
        self.autoloadpath = str(es.ServerVar('eventscripts_addondir')) + "/_autoload/autoload_gungame4.cfg"
        self.errors = {}
    def testApproved(self):
        if self.installer.isScriptApproved() == False:
            self.errors["testApproved"] = "Error: isScriptApproved did not approve gungame4"
            es.dbgmsg(0, "FAILED")
    def testUnApproved(self):
        if self.mminstaller.isScriptApproved() == True:
            self.errors["testUnApproved"] = "Error: isScriptApproved falsely approved mugmod"
            es.dbgmsg(0, "FAILED")
    def testSiteUrl(self):
        # Check base url setting...
        oldsite = self.installer.getBaseUrl()
        newsite = 'http://testadd.eventscripts.com'
        self.installer.setBaseUrl(newsite)
        if not self.installer.getBaseUrl() == newsite:
            self.errors["testSiteUrl"] = "Error: getBaseUrl returned wrong value"
            es.dbgmsg(0, "FAILED")
        self.installer.setBaseUrl(oldsite)
    def testWrongSiteApproved(self):
        # test if it handles a wrong (TERRIBLE) url...
        oldsite = self.installer.getBaseUrl()
        newsite = 'http://useless.example.com'
        self.installer.setBaseUrl(newsite)
        if self.installer.isScriptApproved() == True:
            self.errors["testWrongSiteApproved"] = "Error: isScriptApproved falsely approved gungame on incorrect URL"
            es.dbgmsg(0, "FAILED")
        self.installer.setBaseUrl(oldsite)
    def testDownloadingFile(self):
        # try downloading a few files
        import os
        import zipfile
        if not self.installer.downloadAddon():
            self.errors["testDownloadingFile: Download Gungame"] = "Error: Gungame4 could not be downloaded"
            es.dbgmsg(0, "FAILED")
        if not os.path.exists(self.temppath + "gungame4.zip"):
            self.errors["testDownloadingFile: Check file location"] = "Error: Downloaded file could not be located"
            es.dbgmsg(0, "FAILED")
        else:
            if not zipfile.is_zipfile(self.temppath + "gungame4.zip"):
                self.errors["testDownloadingFile: Check file is a Zip"] = "Error: Downloaded file is not a valid zip archive"
                es.dbgmsg(0, "FAILED")
            else:
                # test that a known file is in the zip
                j = zipfile.ZipFile(self.temppath + "gungame4.zip")
                if not 'addons/eventscripts/gungame4/es_gungame4.txt' in j.namelist():
                    self.errors["testDownloadingFile: Check known file is in Zip archive"] = "Error: Known file cannot be located in Zip archive"
                    es.dbgmsg(0, "FAILED")
        
        if self.gg99installer.downloadAddon():
            self.errors["testDownloadingFile: Download incorrect file"] = "Error: downloadAddon returned download success for a non-existant addon"
            es.dbgmsg(0, "FAILED")
        # do a quick clean up
        if os.path.exists(self.temppath + "gungame99.zip"):
            os.remove(self.temppath + "gungame99.zip")
    def testInstallAddon(self):
        # test installing an addon from download
        if not self.installer.downloadAddon():
            self.errors["testInstallAddon: Download Gungame"] = "Error: Gungame4 could not be downloaded"
            es.dbgmsg(0, "FAILED")
        if not self.installer.installFile():
            self.errors["testInstallAddon: Install Gungame"] = "Error: Gungame4 could not be installed"
            es.dbgmsg(0, "FAILED")
        # and a completely bogus trial....
        if self.gg99installer.installFile():
            self.errors["testInstallAddon: Install non existant addon"] = "Error: Non existant addon was successfully installed"
            es.dbgmsg(0, "FAILED")
    def testAutoloadSetting(self):
        # test setting an addon to autoload
        self.installer.setAutoLoad()
        if not os.path.exists(self.autoloadpath):
            self.errors["testAutoloadSetting"] = "Error: Autoload cfg was not created"
            es.dbgmsg(0, "FAILED")
    def testFullInstall(self):
        # test a FULL installtion of gungame4..
        self.clean()
        self.installer.install()
    def clean(self):
        '''
        Cleans all the unit test files
        '''
        if os.path.exists(self.autoloadpath):
            os.remove(self.autoloadpath)
        if not os.path.exists(self.temppath + "gungame4.zip"):
            self.installer.downloadAddon()
        self.installer.uninstall()
        self.installer.cleanTempDir()
        self.installer.cleanAutoload()
    def go(self):
        es.dbgmsg(0, "[Setting up test]")
        self.setUp()
        # clean stuff up right now
        self.clean()
        # run some tests
        es.dbgmsg(0, "[Testing Checking an approved addon]")
        self.testApproved()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Checking an unapproved addon]")
        self.testUnApproved()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Setting the site URL]")
        self.testSiteUrl()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Checking at the wrong URL]")
        self.testWrongSiteApproved()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Downloading some files]")
        self.testDownloadingFile()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Installing an addon]")
        self.testInstallAddon()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[Testing Settign an addon to Autoload]")
        self.testAutoloadSetting()
        es.dbgmsg(0, "[End Test]")
        self.clean()
        es.dbgmsg(0, "[Testing A full installation of GungGame4]")
        self.testFullInstall()
        es.dbgmsg(0, "[End Test]")
        es.dbgmsg(0, "[es_install] Cleaning test files and resetting")
        self.clean()
        # now output the debug code!
        if len(self.errors) > 0:
            es.dbgmsg(0, "[es_install] ################################################")
            es.dbgmsg(0, "[es_install]  Some errors occured during tests")
            es.dbgmsg(0, "[es_install] ################################################")
            for error in self.errors:
                es.dbgmsg(0, "[es_install] ########### Error ##############################")
                es.dbgmsg(0, "[es_install] %s" % str(error))
                es.dbgmsg(0, "[es_install]  - %s" % str(self.errors[error]))
                es.dbgmsg(0, "[es_install] ################################################")
        else:
            es.dbgmsg(0, "[es_install] ################################################")
            es.dbgmsg(0, "[es_install]  Tests completed successfully")
            es.dbgmsg(0, "[es_install] ################################################")

# ========================
# Internal methods
# ========================            
def install_test():
    '''
    Runs a set of installtion tests....
    ''' 
    if esInstall("gungame4").checkInstalled():
        es.dbgmsg(0, "[es_install] You have Gun Game 4 installed on this system, the unit test uses gungame 4 for it's tests and so cannot continue. ")
        es.dbgmsg(0, "[es_install] Please remove gungameand retry")
    else:
        es.dbgmsg(0, "[es_install] Running es_install unit tests")
        tests = InstallTests()
        es.dbgmsg(0, "[es_install] Running es_install unit tests")
        tests.go()
        es.dbgmsg(0, "[es_install] Tests completed")
        
def install_cmd():
    '''
    Called by es_xinstall <args>
    Checks that the addon isnt installed and that it is apoproved by ESAM
    Then downloads it and extracts the files. Voila!
    '''
    count = es.getargc()
    if count < 2:
        es.dbgmsg(0, "Incorrect syntax for es_xinstall: es_install <basename> [optional: autoload]")
    else:
        addon = es.getargv(1)
        if es.getargv(2) == "autoload":
            installer = esInstall(addon,1)
        else:
            installer = esInstall(addon)
        installer.install()
        
def uninstall_cmd():
    '''
    Called by es_xuninstall <args>
    Uninstalls an addon (mostly used by test class)
    '''
    count = es.getargc()
    if count < 2:
        es.dbgmsg(0, "Incorrect syntax for es_xuninstall: es_uninstall <basename>")
    else:
        addon = es.getargv(1)
        installer = esInstall(addon)
        installer.uninstall()      
def ins_cmd():
    '''
    es_install
    '''
    count = es.getargc()
    if count < 2:
        es.dbgmsg(0, "Incorrect syntax for es_install: es_uninstall <basename> [optional: autoload]")
    else:
        es.server.cmd("es es_xinstall %s" % es.getargs()) 
    
def unins_cmd():
    '''
    es_uninstall
    '''
    count = es.getargc()
    if count < 2:
        es.dbgmsg(0, "Incorrect syntax for es_uninstall: es_uninstall <basename>")
    else:
        es.server.cmd("es es_xuninstall %s" % es.getargs()) 

def autoload_addons():
    '''
    Manages autoloading addons set up by es_install
    '''
    load_path = INSTALL_PATH + "/addons/eventscripts/_autoload/"
    for files in os.listdir(load_path):
        if files.startswith("autoload_") and files.endswith(".cfg"):
            es.server.cmd("es_mexec ../addons/eventscripts/_autoload/%s" % str(files)) 
            
def changelog(message):
    logpath = str(es.ServerVar('eventscripts_addondir')) + "/corelib/es_install/installation_log.log"
    if not os.path.exists(logpath):
        fp = open(logpath, "wb")
    else:
        fp = open(logpath, "a")
    es.log("********************* %s" % message)
    fp.write("%s - %s\n" % (time.strftime("%d/%m/%Y %H:%M:%S %p"), message))
    fp.close()
# ========================
# Public methods
# ========================      
def es_install(addon,autoload=None):
    '''
    Returns an esInstall class instance for the specified addon
    '''
    if autoload:
        return esInstall(addon,1)
    else:
        return esInstall(addon)
# ========================
# Events
# ========================                    
def load():
    '''
    Loads the addon (registers the command)
    '''
    if not es.exists("command", "es_xinstall"):
        es.regcmd('es_xinstall', 'corelib/es_install/install_cmd', 'Install an approved ES addon (syntax: es_install <basename> [optional: autoload]')
    if not es.exists("command", "es_xuninstall"):
        es.regcmd('es_xuninstall', 'corelib/es_install/uninstall_cmd', 'Uninstall an approved ES addon (syntax: es_uninstall <basename>')
    if not es.exists("command", "es_install"):
        es.regcmd('es_install', 'corelib/es_install/ins_cmd', 'Install an approved ES addon (syntax: es_install <basename> [optional: autoload]')
    if not es.exists("command", "es_uninstall"):
        es.regcmd('es_uninstall', 'corelib/es_install/unins_cmd', 'Uninstall an approved ES addon (syntax: es_uninstall <basename>')
    if not es.exists("command", "es_install_test"):
        es.regcmd('es_install_test', 'corelib/es_install/install_test', 'Unit testing for es_install')
    if not os.path.exists(DOWNLOAD_PATH):
        os.mkdir(DOWNLOAD_PATH) 
    # annnnnd load the addons we set to autoload
    autoload_addons()
    
      
def unload():
    pass
    


  
