import es
import unittest
from es import server_var

info = es.AddonInfo()
info.name     = "Python Unit Test Script"
info.version  = "1.1"
info.url      = "http://mattie.info/cs"
info.basename = "pyunittest"
info.author   = "Mattie" 

pytests = unittest.TestSuite()

def addTestCase(testcase):
    print "[Tests] Adding testcase: ", testcase.__name__
    ldr = unittest.TestLoader()
    pytests.addTests(ldr.loadTestsFromTestCase(testcase)._tests)

class PyTestOutput():
    def __init__(self, level=0):
      self.level = level
    def write(self, data):
      self.writeln(data)
    def writeln(self, data=''):
      es.dbgmsg(self.level, '[Tests]: %s' % data)

def load():
    runner = unittest.TextTestRunner()
    runner.stream = PyTestOutput()
    runner.run(pytests)

class SvarTests(unittest.TestCase):
  def setUp(self):
    self.oldgravity = int(es.ServerVar("sv_gravity"))
    es.ServerVar("sv_gravity").set(800)
  def tearDown(self):
    es.ServerVar("sv_gravity").set(self.oldgravity) 
  def testSvar(self):
      s = es.ServerVar("sv_gravity")
      text = str(s)
      self.failUnless(text == "800")
      val = int(s)
      self.failUnless(val == 800)
      fl = float(s)
      self.failUnless(fl == 800.0)
      self.failUnless(s.getName() == "sv_gravity")
  def testCoerce(self):
      s = es.ServerVar("sv_gravity")
      s.set(795)
      self.failUnless(s == 795)
      self.failUnless(s == "795")
addTestCase(SvarTests)

class WhileTests(unittest.TestCase):
  def testWhile(self):
    i = 5000
    es.set("myvar", 5000)
    while i:
        i-=1
        es.server.cmd("es_xmath myvar - 1")
    self.failUnless(int(server_var['myvar'])==0)
addTestCase(WhileTests)

    
class VectorTests(unittest.TestCase):
  def testSvar(self):
      y = es.createvectorstring(3.5,4.5,6.8)
      self.failUnless(y == "3.500000,4.500000,6.800000")
      z = es.splitvectorstring(y)
      self.failIf(z[0] != 3.5 or z[1] != 4.5 or round(z[2],1) != 6.8)
addTestCase(VectorTests)

# begin CmdTests section
class CmdTests(unittest.TestCase):
  def setUp(self):
    # lets suppress some noise
    debug = es.ServerVar("eventscripts_debug")
    olddebug = int(debug)
    if int(debug) == 0:
      debug.set(-1)
    es.regcmd("pycommand", "pyunittest/cmdtest_run", "Stuff")
    debug.set(olddebug)
    es.regsaycmd("pycommand","pyunittest/cmdtest_run", "Stuff")
    es.regclientcmd("pycommand","pyunittest/cmdtest_run", "Stuff")
  def tearDown(self):
    es.unregsaycmd("pycommand")
    es.unregclientcmd("pycommand")
  def testCmd(self):
      es.server.cmd("pycommand passed")
      self.failUnless(server_var['cmdtest_pass'] == "passed")
      self.failUnless(server_var['cmdtest_pass2'] == "passed")
addTestCase(CmdTests)
    
def cmdtest_run():
    es.set("cmdtest_pass", es.getargv(1))
    #print "cmdtest(), args: %s, argc: %d" % (es.getargs(), es.getargc())
    if es.getargc() != 2:
      print "cmdtest0_failed,", es.getargc()
    es.doblock("pyunittest/cmdtest_block")

def cmdtest_block():
    es.set("cmdtest_pass2", es.getargv(1))
# end CmdTests section

class KeyTests(unittest.TestCase):
  def setUp(self):
      pass
  def tearDown(self):
      pass
  def keytest2(self, x):
      self.failIf(x <= 0)
      name = es.keypgetname(x)
      self.failIf(name != "justice")
      es.keypsetname(x, "justice2")
      name = es.keypgetname(x)
      self.failIf(name != "justice2")
      y = es.keypfindsubkey(x, "subkey", False)
      self.failUnless(y is None)
      y = es.keypfindsubkey(x, "subkey", True)
      subname = es.keypgetname(y)
      self.failUnless(subname == "subkey")
      r = es.keypfindsubkey(x, "subkey", True)
      self.failUnless(r==y)
      q = es.keypfindsubkey(x, "subkey2", True)
      subname = es.keypgetname(q)
      self.failIf(subname != "subkey2")
      w = es.keypfindsubkey(x, "subkey3", True)
      e = es.keypfindsubkey(x, "subkey3", True)
      p = es.keypgetfirstsubkey(x)
      n = 0
      while p:
        n+=1
        p = es.keypgetnextkey(p)
      self.failUnless(n==3)
      self.failUnless(w==e)
      es.keypdetachsubkey(x, w)
      e = es.keypfindsubkey(x, "subkey3", False)
      self.failUnless(e is None)
      es.keypdelete(w)
      p = es.keypgetfirstsubkey(x)
      n = 0
      while p:
        n+=1
        p = es.keypgetnextkey(p)
      self.failUnless(n==2)
      self.failUnless(es.keypisempty(r))
      # setstring    
      es.keypsetstring(r, "hello", "1.3")
      self.failIf(es.keypisempty(r))
      outcome = es.keypgetstring(y, "hello")
      self.failUnless(outcome=="1.3")
      outcome = es.keypgetint(y, "hello")
      self.failUnless(outcome == 1)
      outcome = es.keypgetfloat(y, "hello")
      self.failUnless(round(outcome,1) == 1.3)
      # setint
      es.keypsetint(y, "hello", 99)
      outcome = es.keypgetint(y, "hello")
      self.failUnless(outcome==99)
      # setfloat
      es.keypsetint(y, "hello", 99)
      outcome = es.keypgetint(y, "hello")
      self.failUnless(outcome==99)
      # save to file
      es.keypsavetofile(x, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")

  def keytest3(self,x):
      es.keyploadfromfile(x, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")
      y = es.keypfindsubkey(x, "subkey", False)
      outcome = es.keypgetint(y, "hello")
      self.failUnless(outcome==99)
      z = es.keypfindsubkey(y, "test", True)
      name = es.keypgetname(z)
      self.failUnless(name =="test")
      pp = es.keypcreate()
      es.keyploadfromfile(pp, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")
      es.keyprecursivekeycopy(z, pp)
      name = es.keypgetname(z)
      self.failUnless(name == "justice2")

  def testKeygroup(self):
      es.keygroupcreate("justice")
      x = es.keygroupgetpointer("justice")
      self.keytest2(x)
      es.keygroupdelete("justice2")
      es.keygroupcreate("justice2")
      x = es.keygroupgetpointer("justice2")
      self.keytest3(x)
      import keyvalues
      y = keyvalues.getKeyGroup("justice2")
      es.keygroupdelete("justice2")

  def testNewkey(self):
      x = es.keypcreate()
      es.keypsetname(x, "justice")
      self.keytest2(x)
      es.keypdelete(x)
      x = es.keypcreate()
      es.keypsetname(x, "justice2")
      self.keytest3(x)
      es.keypdelete(x)
      return
addTestCase(KeyTests)

class CreateScriptListTest(unittest.TestCase):
  def testCreatescriptlist(self):
      j = es.createscriptlist()
      self.failUnless(j and len(j))
      j = es.createscriptlist("xxxxxxxxlarry")
      self.failIf(j or len(j))
      j = es.createscriptlist('corelib')
      self.failUnless(len(j)==1)
addTestCase(CreateScriptListTest)

class MaxPlayerTests(unittest.TestCase):
  def testGetmaxplayers(self):
      if ('' != es.ServerVar('eventscripts_currentmap')):
          j = es.getmaxplayercount()
          self.failUnless(j)
          # in CS:S we know the max players is cs_team_manager index -1 
          if es.getgame() == "Counter-Strike: Source":
            q = es.getentityindex('cs_team_manager') - 1
            if q:
              self.failUnless(q==j)
      else:
          print "Warning: Disabled MaxPlayerTests because map not loaded."
addTestCase(MaxPlayerTests)

class MoreWhileTests(unittest.TestCase):
  def testWhile(self):
    i = 5000
    myvar = es.ServerVar('myvar', 5000)
    myvar2 = es.ServerVar('myvar2', 0)
    myvar3 = es.ServerVar('myvar2', 0)    
    while i:
        i-=1
        es.server.cmd("es_xmath myvar2 + 1;es_xmath myvar3 + 1;es_xmath myvar - 1")
    self.failUnless(int(myvar)==0)
    self.failUnless(int(myvar2)==5000)
    self.failUnless(int(myvar3)==5000)
addTestCase(WhileTests)

class ServerCmdTest(unittest.TestCase):
  def testServerCmd(self):
      es.server.cmd("es_xset __servercmd 33")
      p = int(es.server_var['__servercmd'])
      self.failUnless(p == 33)
  def testNonExistentServerCmd(self):
      es.server.cmd("es______xset __servercmd 33")
      self.failUnless(True)
addTestCase(ServerCmdTest)

class EntityPropTest(unittest.TestCase):
  def setUp(self):
      self.runme = False
      if ('' != es.ServerVar('eventscripts_currentmap')):
          self.runme = True
          es.trick("entity", "logic_choreographed_scene")
          self.index = int(es.ServerVar('eventscripts_lastgive'))
      else:
          print "Warning: Disabled EntityPropTest because map not loaded."
  def tearDown(self):
      if self.runme:
          # TODO: kill logic_choreographed_scene
          pass      
  def testEntityProp(self):
      if self.runme:
          prop = es.EntityProp(self.index, "CSceneEntity.m_bMultiplayer")
          self.failUnless(int(prop) == 0)
          prop.set(3)
          print prop
          self.failUnless(int(prop) == 3)
  def testCreateEntityIndexList(self):
      if self.runme:
          d1 = es.createentityindexlist()
          d2 = es.createentitylist()
          # check sizes              
          self.failUnless(len(d2) == len(d1))
          self.failUnless(len(d1) > 0)          
          # be sure the data is correct for that index
          self.failUnless(d1[self.index]["classname"]=="logic_choreographed_scene")
          self.failUnless(d2[self.index]["classname"]=="logic_choreographed_scene")
          # create only items of that name
          d3 = es.createentityindexlist("logic_choreographed_scene")
          self.failUnless(d3[self.index]["classname"]=="logic_choreographed_scene")
          for j in d3.keys():
              self.failUnless(d3[j]["classname"]=="logic_choreographed_scene")
          d4 = es.createentityindexlist("player")
          self.failIf(d4.has_key(self.index))
addTestCase(EntityPropTest)

class ServerCmdProxyTest(unittest.TestCase):
  def testServerCmdProxy(self):
      es.set("__servercmd", 1)
      es.server.es_xset("__servercmd", 33)
      p = int(es.server_var['__servercmd'])
      self.failUnless(p == 33)
  def testServerCmdProxyMultiArguments(self):
      es.set("__servercmd", 1)
      es.server.es_xset("__servercmd", 33, "This is a fake description")
      p = int(es.server_var['__servercmd'])
      self.failUnless(p == 33)
  def testNonExistentServerCmdProxy(self):
      try:
          es.server.es______xset("__servercmd", 33)
      except AttributeError:
          return
      self.failUnless(False)
addTestCase(ServerCmdProxyTest)
